-- Configuration file for Ateles (Periodic Oscillator) --

-- This is a configuration file for the Finite Volume / Discontinuous Galerkin Solver ATELES. 
-- It provides a testcase for the simulation of Acoustic equations in a homogenous media. The simulation domain
-- is a periodic cube with edge length 2.0. Therefore this is a very good way to verify your algorithmic implementations, 
-- since this testcase does not involve any boundary conditions. 
-- The testcase simulates the temporal development of standing waves for acoustic equation. Since we 
-- are considering a very simple domain, an analytic solution is well known and given as Lua functions in this script.
-- Therefore we suggest this testcase to verify one of the following topics
-- ... algorihtmic correctness
-- ... spatial and temporal order of your scheme
-- ... diffusion and dispersion relations of your scheme
-- ... and many more.
-- To verify diffusion and dispersion relations this testcases allows you to modify the spatial harmonics by
-- varying the integer mode number in x and y direction by modifying the lua variables m and n.
-- This testcase can be run in serial (only one execution unit) or in parallel (with multiple mpi ranks).
-- To calculate a grid convergence behavior please modify the level variable. An increment of one will half the radius of your elements.

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- Parameters to vary --
require 'seeder_right'
degree = 7
poly_space = 'Q'

timestep_info = 100

-- Check for Nans and unphysical values
check =  {
           interval = 100,
         }

period=1
--...Configuration of simulation time
simulation_name='ateles_right' 
sim_control = { 
                time_control = { max = period*0.007,  -- final Simulated time
                                 min = 0,
                                 interval = {iter = 100} 
                                }
              }

--...Configuration for the restart file
restart = { 
--            -- file to restart from
--            read = './restart/maxwell/per_osci_maxwell_modg_lastHeader.lua',                        
--            -- folder to write restart data to
            write = 'restart/right/',
            -- temporal definition of restart write
            time_control= { 
                            min = 0,
                            interval = sim_control.time_control.max/25.0,
                   },
          }

-- table for preCICE
precice = {
           accessor = 'Ateles_right',
           configFile ='config_twoway_scalar_cps.xml',
          }

-- End  Parameters to vary --
--------------------------------------------------------------------------------
-- Definition of the test-case.

-- Mesh definitions --
mesh = 'mesh_right/'

-- Tracking              
eps=bounding_cube.length/math.pow(2,level+1)
racking = {
             label = 'track_line_density_m7',
             folder = './',
             variable = {'density'},
             shape = {kind = 'canoND', object= { origin = {0.0+eps,0.0,0.0}, 
                                               }
                     },
             time_control= { min = 0,
                           --  max = sim_control.time_control.max,
                             interval = sim_control.time_control.max/8.0,
                           },
             format = 'ascii',
           }


-- Global simulation options
simulation = {
              name = 'plane_wave_X'..poly_space..(degree+1),            -- the name of the simualtion
              time = simtime
             }

-- Equation definitions --
equation = {
             name   = 'acoustic',
             background = {
                 density = 1.225, 
                 velocityX = 0.0,
                 velocityY = 0.0,
                 velocityZ = 0.0,
                 pressure = 100000.0
                 }
           }

-- Scheme definitions --
scheme = {
    -- the spatial discretization scheme
    spatial =  {
               name = 'modg',            -- we use the modal discontinuous Galerkin scheme 
               m =  degree,                   -- the maximal polynomial degree for each spatial direction
               modg_space = poly_space
               }, 
    -- the temporal discretization scheme
    temporal = {
                 name = 'explicitRungeKutta', 
                 steps = 4,
              -- how to control the timestep
                 control = {
                          name = 'cfl',   -- the name of the timestep control mechanism
                          cfl  = 0.95,     -- CourantÐFriedrichsÐLewy number
                         },
               },
             }

-- ...the general projection table
projection = {
              kind = 'fpt',  -- 'fpt' or 'l2p', default 'l2p'
              -- for fpt the  nodes are automatically 'chebyshev'
              -- for lep the  nodes are automatically 'gauss-legendre'
           -- lobattopoints = false  -- if lobatto points should be used, default = false,
                                     -- only working for chebyshev points --> fpt
              factor = 1.0,          -- dealising factpr for fpt, oversampling factor for l2p, float, default 1.0
           -- blocksize = 32,        -- for fpt, default -1
           -- fftmultithread = false -- for fpt, logical, default false
             }


c = math.sqrt(equation.background.pressure / equation.background.density)
ampl_density= (equation.background.density/c) 
ampl_X= ampl_density /equation.background.density * ( equation.background.velocityX - c)
ampl_Y = 1.0--0.001225
ampl_Z = 1.0 --0.001225

function velocityX(x,y,z,t)
  return ( (ampl_X * math.cos(math.pi*x-0.5-c*t)) )
end 
function velocityY(x,y,z,t)
  return ( (ampl_Y * math.cos(math.pi*y-c*t)) )
end 
function velocityZ(x,y,z,t)
  return ( (ampl_Z * math.cos(math.pi*z-c*t)) )
end 
function density(x,y,z,t)
  return ( ampl_density * math.cos(math.pi*x-0.5-c*t) )
end 

function init_velocityX(x,y,z)
  return (velocityX(x,y,z,0.0)  )
end 
function init_velocityY(x,y,z)
  return (velocityY(x,y,z,0.0)  )
end 
function init_velocityZ(x,y,z)
  return (velocityZ(x,y,z,0.0)  )
end 
function init_density(x,y,z)
  return ( density(x,y,z,0.0) )
end 

-- Initial Condition definitions --
initial_condition = { 
                      density = 0.0, --init_density, 
                      velocityX = 0.0, --init_velocityX,
                      velocityY = 0.0,
                      velocityZ = 0.0,
                    }
 -- Boundary definitions
 boundary_condition = {
                         {
                           label = 'wall_1',
                           kind = 'outflow',
                           density = '0.0'
                         }
                         ,
                         {
                           label = 'wall_2',
                           kind = 'outflow',
                           density = '0.0'
                         }
                         ,
                         {
                           label = 'wall_4',
                           kind = 'outflow',
                           density = '0.0'
                         }
                         ,
                         {
                           label = 'wall_5',
                           kind = 'outflow',
                           density = '0.0'
                         }
                         ,
                         {
                           label = 'wall_6',
                           kind = 'outflow',
                           density = '0.0'
                         }
                         ,
                         {
                           label = 'precice_rightmesh',
                           kind = 'precice',
                           precice_mesh = 'AcousticSurface',
                           provide_mesh = false,
                           exchange_data_write = {'Density_right', 
                                                  'Vel_X_right', 
                                                  'Vel_Y_right', 
                                                  'Vel_Z_right'
                                                  },
                           exchange_data_read =  {'Density_left', 
                                                  'Vel_X_left', 
                                                  'Vel_Y_left', 
                                                  'Vel_Z_left'
                                                  }
                         }
                         ,
                       }

precice_write = {  label = {'precice_rightmesh'} }
